//+------------------------------------------------------------------+
//|                                                  PanelDialog.mqh |
//|                        Copyright 2011, MetaQuotes Software Corp. |
//|                                              http://www.mql5.com |
//+------------------------------------------------------------------+
#include <Controls\Dialog.mqh>
#include <Controls\Panel.mqh>
#include <Controls\Edit.mqh>
#include <Controls\Label.mqh>
#include <Controls\SpinEdit.mqh>
//+------------------------------------------------------------------+
//| defines                                                          |
//+------------------------------------------------------------------+
//---   
#define INDENT_LEFT                         (11)      //    (   )
#define INDENT_TOP                          (11)      //   (   )
#define INDENT_RIGHT                        (11)      //   (   )
#define INDENT_BOTTOM                       (11)      //    (   )
#define CONTROLS_GAP_X                      (10)      //   X
#define CONTROLS_GAP_Y                      (10)      //   Y
//---  
#define LABEL_WIDTH                         (50)      //   X
//---  
#define EDIT_WIDTH                          (50)      //   X
#define EDIT_HEIGHT                         (20)      //   Y
//---    (RGB)
#define BASE_COLOR_MIN                      (0)       //     
#define BASE_COLOR_MAX                      (255)     //     
//+------------------------------------------------------------------+
//|  CPanelDialog                                               |
//| :                               |
//+------------------------------------------------------------------+
class CPanelDialog : public CAppDialog
  {
private:
   //---  
   CPanel            m_color;                         //    
   CLabel            m_label_red;                     //    ""
   CEdit             m_field_red;                     //    ""
   CLabel            m_label_green;                   //    ""
   CEdit             m_field_green;                   //    ""
   CLabel            m_label_blue;                    //    ""
   CSpinEdit         m_edit_blue;                     //    ""
   //---  
   int               m_red;                           //  ""
   int               m_green;                         //  ""
   int               m_blue;                          //  ""

public:
                     CPanelDialog(void);
                    ~CPanelDialog(void);
   //--- 
   virtual bool      Create(const long chart,const string name,const int subwin,const int x1,const int y1,const int x2,const int y2);
   //---   
   virtual bool      OnEvent(const int id,const long &lparam,const double &dparam,const string &sparam);
   //--- 
   void              SetRed(const int value);
   void              SetGreen(const int value);
   void              SetBlue(const int value);

protected:
   //---   
   bool              CreateColor(void);
   bool              CreateRed(void);
   bool              CreateGreen(void);
   bool              CreateBlue(void);
   //---    
   void              OnChangeBlue(void);
   //--- 
   void              SetColor(void);
  };
//+------------------------------------------------------------------+
//|                                                  |
//+------------------------------------------------------------------+
EVENT_MAP_BEGIN(CPanelDialog)
   ON_EVENT(ON_CHANGE,m_edit_blue,OnChangeBlue)
EVENT_MAP_END(CAppDialog)
//+------------------------------------------------------------------+
//|                                                       |
//+------------------------------------------------------------------+
CPanelDialog::CPanelDialog(void) : m_red((BASE_COLOR_MAX-BASE_COLOR_MIN)/2),
                                   m_green((BASE_COLOR_MAX-BASE_COLOR_MIN)/2),
                                   m_blue((BASE_COLOR_MAX-BASE_COLOR_MIN)/2)
  {
  }
//+------------------------------------------------------------------+
//|                                                        |
//+------------------------------------------------------------------+
CPanelDialog::~CPanelDialog(void)
  {
  }
//+------------------------------------------------------------------+
//|                                                          |
//+------------------------------------------------------------------+
bool CPanelDialog::Create(const long chart,const string name,const int subwin,const int x1,const int y1,const int x2,const int y2)
  {
//---    
   if(!CAppDialog::Create(chart,name,subwin,x1,y1,x2,y2))                           return(false);
//---   
   if(!CreateColor())                                                               return(false);
   if(!CreateRed())                                                                 return(false);
   if(!CreateGreen())                                                               return(false);
   if(!CreateBlue())                                                                return(false);
//---   
   SetColor();
//--- 
   return(true);
  }
//+------------------------------------------------------------------+
//|                                             |
//+------------------------------------------------------------------+
bool CPanelDialog::CreateColor(void)
  {
//--- 
   int x1=INDENT_LEFT+LABEL_WIDTH+CONTROLS_GAP_X+EDIT_WIDTH+CONTROLS_GAP_X;
   int y1=INDENT_TOP;
   int x2=ClientAreaWidth()-INDENT_RIGHT;
   int y2=ClientAreaHeight()-INDENT_BOTTOM;
//--- 
   if(!m_color.Create(m_chart_id,m_name+"Color",m_subwin,x1,y1,x2,y2))              return(false);
   if(!m_color.ColorBorder(CONTROLS_EDIT_COLOR_BORDER))                             return(false);
   if(!Add(m_color))                                                                return(false);
   m_color.Alignment(WND_ALIGN_WIDTH,x1,0,INDENT_RIGHT,0);
//--- 
   return(true);
  }
//+------------------------------------------------------------------+
//|    "Red"             |
//+------------------------------------------------------------------+
bool CPanelDialog::CreateRed(void)
  {
//--- 
   int x1=INDENT_LEFT;
   int y1=INDENT_TOP;
   int x2=x1+EDIT_WIDTH;
   int y2=y1+EDIT_HEIGHT;
//---  
   if(!m_label_red.Create(m_chart_id,m_name+"LabelRed",m_subwin,x1,y1+1,x2,y2))     return(false);
   if(!m_label_red.Text("Red"))                                                     return(false);
   if(!Add(m_label_red))                                                            return(false);
//---  
   x1+=LABEL_WIDTH+CONTROLS_GAP_X;
   x2=x1+EDIT_WIDTH;
//---  
   if(!m_field_red.Create(m_chart_id,m_name+"Red",m_subwin,x1,y1,x2,y2))            return(false);
   if(!m_field_red.Text(IntegerToString(m_red)))                                    return(false);
   if(!m_field_red.ReadOnly(true))                                                  return(false);
   if(!Add(m_field_red))                                                            return(false);
//--- 
   return(true);
  }
//+------------------------------------------------------------------+
//|    "Green"           |
//+------------------------------------------------------------------+
bool CPanelDialog::CreateGreen(void)
  {
//--- 
   int x1=INDENT_LEFT;
   int y1=INDENT_TOP+(EDIT_HEIGHT+CONTROLS_GAP_Y);
   int x2=x1+EDIT_WIDTH;
   int y2=y1+EDIT_HEIGHT;
//---  
   if(!m_label_green.Create(m_chart_id,m_name+"LabelGreen",m_subwin,x1,y1+1,x2,y2)) return(false);
   if(!m_label_green.Text("Green"))                                                 return(false);
   if(!Add(m_label_green))                                                          return(false);
//---  
   x1+=LABEL_WIDTH+CONTROLS_GAP_X;
   x2=x1+EDIT_WIDTH;
//---  
   if(!m_field_green.Create(m_chart_id,m_name+"Green",m_subwin,x1,y1,x2,y2))        return(false);
   if(!m_field_green.Text(IntegerToString(m_green)))                                return(false);
   if(!m_field_green.ReadOnly(true))                                                return(false);
   if(!Add(m_field_green))                                                          return(false);
//--- 
   return(true);
  }
//+------------------------------------------------------------------+
//|    "Blue"           |
//+------------------------------------------------------------------+
bool CPanelDialog::CreateBlue(void)
  {
//--- 
   int x1=INDENT_LEFT;
   int y1=INDENT_TOP+2*(EDIT_HEIGHT+CONTROLS_GAP_Y);
   int x2=x1+EDIT_WIDTH;
   int y2=y1+EDIT_HEIGHT;
//---  
   if(!m_label_blue.Create(m_chart_id,m_name+"LabelBlue",m_subwin,x1,y1+1,x2,y2))   return(false);
   if(!m_label_blue.Text("Blue"))                                                   return(false);
   if(!Add(m_label_blue))                                                           return(false);
//---  
   x1+=LABEL_WIDTH+CONTROLS_GAP_X;
   x2=x1+EDIT_WIDTH;
//---  
   if(!m_edit_blue.Create(m_chart_id,m_name+"Blue",m_subwin,x1,y1,x2,y2))           return(false);
   if(!Add(m_edit_blue))                                                            return(false);
   m_edit_blue.MinValue(BASE_COLOR_MIN);
   m_edit_blue.MaxValue(BASE_COLOR_MAX);
   m_edit_blue.Value(m_blue);
//--- 
   return(true);
  }
//+------------------------------------------------------------------+
//|   "Red"                                         |
//+------------------------------------------------------------------+
void CPanelDialog::SetRed(const int value)
  {
//--- 
   if(value<0 || value>255) return;
//--- 
   m_red=value;
//--- 
   m_field_red.Text(IntegerToString(value));
//---   
   SetColor();
  }
//+------------------------------------------------------------------+
//|   "Green"                                       |
//+------------------------------------------------------------------+
void CPanelDialog::SetGreen(const int value)
  {
//--- 
   if(value<0 || value>255) return;
//--- 
   m_green=value;
//--- 
   m_field_green.Text(IntegerToString(value));
//---   
   SetColor();
  }
//+------------------------------------------------------------------+
//|   "Blue"                                        |
//+------------------------------------------------------------------+
void CPanelDialog::SetBlue(const int value)
  {
//--- 
   if(value<0 || value>255) return;
//--- 
   m_blue=value;
//--- 
   m_edit_blue.Value(value);
//---   
   SetColor();
  }
//+------------------------------------------------------------------+
//|     ""                     |
//+------------------------------------------------------------------+
void CPanelDialog::OnChangeBlue(void)
  {
//--- 
   m_blue=m_edit_blue.Value();
//---   
   SetColor();
  }
//+------------------------------------------------------------------+
//|                                              |
//+------------------------------------------------------------------+
void CPanelDialog::SetColor(void)
  {
   m_color.ColorBackground(m_red+(m_green<<8)+(m_blue<<16));
  }
//+------------------------------------------------------------------+
